USE MASTER
GO

EXEC SP_CONFIGURE 'ALLOW UPDATES', 1
RECONFIGURE WITH OVERRIDE
GO

/* =============================================================*/
/*   Function: bin_to_hex                                       */
/* =============================================================*/

/* Drop the function if it already exists then create the function */
IF EXISTS ( SELECT 1 FROM information_schema.routines WHERE routine_name = 'fn_bin_to_hex' AND routine_schema = 'system_function_schema' )
  DROP FUNCTION system_function_schema.fn_bin_to_hex
GO

/*** Function: fn_bin_to_hex ******************************/
/* Description:                                           */
/*   Converts a 16 digit binary string into a hex string  */
/*   string.                                              */
/*   If the input format isn't correct the input (up to   */
/*   255 characters) is returned.                         */
/*                                                        */
/* Input:                                                 */
/*   varchar(255)                                         */
/*                                                        */
/* Output:                                                */
/*   varchar(255)                                         */
/*                                                        */
/* Developer                                   Date       */
/*   Notes                                                */
/* -------------------------------             ---------- */
/* Bri Gipson                                  11-28-2007 */
/*   Initial Creation                                     */
/**********************************************************/
CREATE FUNCTION system_function_schema.fn_bin_to_hex ( @hex_text VARBINARY(255) )
RETURNS VARCHAR(255)
AS
BEGIN 
  DECLARE @result VARCHAR(255)
  DECLARE @HexStr CHAR(16)

  SELECT @result = @hex_text
  SELECT @HexStr = '0123456789ABCDEF'

  IF LEN( @hex_text ) = 16
  BEGIN
    SELECT @result = 
      '0x' +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 1, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 1, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 2, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 2, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 3, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 3, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 4, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 4, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 5, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 5, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 6, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 6, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 7, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 7, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 8, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 8, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 9, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 9, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 10, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 10, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 11, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 11, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 12, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 12, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 13, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 13, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 14, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 14, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 15, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 15, 1 ) / 1 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 16, 1 ) / 16 ) % 16 + 1 , 1 ) +
      SUBSTRING( @HexStr, ( SUBSTRING( @hex_text, 16, 1 ) / 1 ) % 16 + 1 , 1 )
  END

  RETURN @result
END
GO

EXEC SP_CONFIGURE 'ALLOW UPDATES', 0
RECONFIGURE WITH OVERRIDE
GO

/*
-- Calling the function source from SQL Server:
DECLARE @ObjectID int -- the object id

SELECT @ObjectID = id
FROM master..sysobjects
WHERE name = 'fn_bin_to_hex'

SELECT
  [text]
FROM
  master..syscomments
WHERE
  id IN
  (
    SELECT
      id
    FROM
      master..sysobjects
    WHERE
      name = 'fn_bin_to_hex'
  )

-- Calling the function in a SELECT statement:
-- SELECT fn_bin_to_hex( 0x41223344556677889900aabbccddeeff )
*/
