/*
  converts a binary(16) value into a hexadecimal string:
  0x################... or
  ##############...

  input arguments:
  string representation of the hex (string)

  output:
  data (string) - Binary and text values are sent as string
  output will always contain the leading "0x"

  registering the function:
  CREATE FUNCTION fn_bin_to_hex RETURNS STRING SONAME 'udf_bin_to_hex.so';
  ( use .dll instead of .so on Windows environments. )

  getting rid of the function:
  DROP FUNCTION fn_bin_to_hex;

  testing the function:
  SELECT fn_bin_to_hex( '0123456789ABCDEF' );
*/

// Download mysql-5.0.26-win-src.zip at mySQL.org
// Reference the include directory from:
// Within Solution Explorer, right-click HexToBinUDF and select properties and go to the "General" section.
// For "Additional Include Directories". On my local environment, the path is "C:\mysql-5.0.26-win-src\mysql-5.0.26\include"

/*
gcc -shared -lstdc++ -I /usr/include -I /usr/local/include -I /usr/local/mysql/include/mysql -o udf_bin_to_hex.so udf_bin_to_hex.cc
cp udf_bin_to_hex.so /usr/lib
*/


#ifdef STANDARD
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#ifdef __WIN__
typedef unsigned __int64 ulonglong;	
typedef __int64 longlong;
#else
typedef unsigned long long ulonglong;
typedef long long longlong;
#endif /*__WIN__*/
#else
#include <my_global.h>
#include <my_sys.h>
#endif
#include <mysql.h>
#include <m_ctype.h>
#include <m_string.h>


extern "C" 
{
  my_bool fn_bin_to_hex_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
  void fn_bin_to_hex_deinit(UDF_INIT *initid);
  char *fn_bin_to_hex(UDF_INIT *initid, UDF_ARGS *args, char *result, unsigned long *length, char *is_null, char *error);
}

char bytestring_to_character( char nibble_a, char nibble_b )
{
  char result;

  result = ( ( ( ( nibble_a & 0x40 ) >> 6 ) * 9 ) + ( nibble_a & 0x0f ) ) * 16 + // / 64
           ( ( ( ( nibble_b & 0x40 ) >> 6 ) * 9 ) + ( nibble_b & 0x0f ) );

  return result;
}

bool is_dash( char input )
{
  return ( input != 0x2D );
}

bool is_hex( char input )
{
  return ( ( ( ( input & 0xF0 ) == 0x30 ) && ( ( input & 0x0F ) < 0x0A ) ) || ( ( ( input & 0xD8 ) == 0x40 ) && ( ( input & 0x07 ) != 0x07 ) ) );
}

int validate_hex_format( char *input )
{
  int x;

  for ( x = 0; x < 32; x++ )
  {
    // Make sure we only use values 0-9|A-F|a-f
    if ( !is_hex( input[x] ) )
    {
      return ( x + 1 );
    }
  }

  return 0;
}

my_bool fn_bin_to_hex_init( UDF_INIT *initid, UDF_ARGS *args, char *result)
{
  /*
    Because of a request to return the input if the input is invalid (instead of a null)
    we cannot say that we have a max_length or that our result may be null anymore.
  */
  initid->const_item = 0;
  initid->maybe_null = 0;
  /*
  initid->max_length = 16;
  initid->maybe_null = 1;
  */

  // sprintf( result, "fn_bin_to_hex: arg_count=%d lengths[0]=%d \"%s\".",
  //   args->arg_count, args->lengths[0], args->args[0] );
  // return 1;

  if ( args->arg_count != 1 || args->arg_type[0] != STRING_RESULT )
  {
    strcpy( result, "fn_bin_to_hex: Wrong argument type. Expected a hex string." );
    return 1;
  }
  /* 
    It was requested that if a non-hex string was sent to just return
    the input as the result.
    That means we don't need to check the argument size here, but later.
  else
  if ( args->lengths[0] != 36 )
  {
    sprintf( result, "fn_bin_to_hex: Wrong argument size. Expected 36 characters. Received %d character(s).", args->lengths[0] );
    return 1;
  }
  */
  // memcpy( input, args->args[0], 36 );

  return 0;
}

void fn_bin_to_hex_deinit(UDF_INIT *initid)
{
  // Nothing to deinitialize. Nothing to deallocate.
}

char *fn_bin_to_hex( UDF_INIT *initid, UDF_ARGS *args, char *result, unsigned long *length, char *is_null, char *error )
{
  char word_in[16];
  char word_out[34];

  /*
    Because of the request mentioned in _init, an if-then-else clause was added here.
    If the length is not 16 characters long, just return what was sent. Otherwise,
    interpret it as a binary string and convert it to a GUID.
  */
  if ( args->lengths[0] != 16 )
  {
    memcpy( result, args->args[0], args->lengths[0] );
    *length = args->lengths[0];
    return result;
  }

  memcpy( word_in, args->args[0], 16 );

  word_out[0] = '0';
  word_out[1] = 'x';
  word_out[2] = "0123456789ABCDEF"[ ( word_in[0] & 0xF0 ) >> 4 ];
  word_out[3] = "0123456789ABCDEF"[ word_in[0] & 0x0F ];
  word_out[4] = "0123456789ABCDEF"[ ( word_in[1] & 0xF0 ) >> 4 ];
  word_out[5] = "0123456789ABCDEF"[ word_in[1] & 0x0F ];
  word_out[6] = "0123456789ABCDEF"[ ( word_in[2] & 0xF0 ) >> 4 ];
  word_out[7] = "0123456789ABCDEF"[ word_in[2] & 0x0F ];
  word_out[8] = "0123456789ABCDEF"[ ( word_in[3] & 0xF0 ) >> 4 ];
  word_out[9] = "0123456789ABCDEF"[ word_in[3] & 0x0F ];
  word_out[10] = "0123456789ABCDEF"[ ( word_in[4] & 0xF0 ) >> 4 ];
  word_out[11] = "0123456789ABCDEF"[ word_in[4] & 0x0F ];
  word_out[12] = "0123456789ABCDEF"[ ( word_in[5] & 0xF0 ) >> 4 ];
  word_out[13] = "0123456789ABCDEF"[ word_in[5] & 0x0F ];
  word_out[14] = "0123456789ABCDEF"[ ( word_in[6] & 0xF0 ) >> 4 ];
  word_out[15] = "0123456789ABCDEF"[ word_in[6] & 0x0F ];
  word_out[16] = "0123456789ABCDEF"[ ( word_in[7] & 0xF0 ) >> 4 ];
  word_out[17] = "0123456789ABCDEF"[ word_in[7] & 0x0F ];
  word_out[18] = "0123456789ABCDEF"[ ( word_in[8] & 0xF0 ) >> 4 ];
  word_out[19] = "0123456789ABCDEF"[ word_in[8] & 0x0F ];
  word_out[20] = "0123456789ABCDEF"[ ( word_in[9] & 0xF0 ) >> 4 ];
  word_out[21] = "0123456789ABCDEF"[ word_in[9] & 0x0F ];
  word_out[22] = "0123456789ABCDEF"[ ( word_in[10] & 0xF0 ) >> 4 ];
  word_out[23] = "0123456789ABCDEF"[ word_in[10] & 0x0F ];
  word_out[24] = "0123456789ABCDEF"[ ( word_in[11] & 0xF0 ) >> 4 ];
  word_out[25] = "0123456789ABCDEF"[ word_in[11] & 0x0F ];
  word_out[26] = "0123456789ABCDEF"[ ( word_in[12] & 0xF0 ) >> 4 ];
  word_out[27] = "0123456789ABCDEF"[ word_in[12] & 0x0F ];
  word_out[28] = "0123456789ABCDEF"[ ( word_in[13] & 0xF0 ) >> 4 ];
  word_out[29] = "0123456789ABCDEF"[ word_in[13] & 0x0F ];
  word_out[30] = "0123456789ABCDEF"[ ( word_in[14] & 0xF0 ) >> 4 ];
  word_out[31] = "0123456789ABCDEF"[ word_in[14] & 0x0F ];
  word_out[32] = "0123456789ABCDEF"[ ( word_in[15] & 0xF0 ) >> 4 ];
  word_out[33] = "0123456789ABCDEF"[ word_in[15] & 0x0F ];

  memcpy( result, word_out, 34);
  *length = (unsigned long)34;

  return result;
}
